@@ -1,6 +1,6 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  *   Mupen64plus-core - api/frontend.c                                     *
- *   Mupen64Plus homepage: http://code.google.com/p/mupen64plus/           *
+ *   Mupen64Plus homepage: https://mupen64plus.org/                        *
  *   Copyright (C) 2012 CasualJames                                        *
  *   Copyright (C) 2009 Richard Goedeken                                   *
  *                                                                         *
@@ -19,7 +19,7 @@
  *   Free Software Foundation, Inc.,                                       *
  *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.          *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-                       
+
 /* This file contains the Core front-end functions which will be exported
  * outside of the core library.
  */
@@ -28,6 +28,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <md5.h>
 
 #define M64P_CORE_PROTOTYPES 1
 #include "callbacks.h"
@@ -38,19 +39,21 @@
 #include "main/cheat.h"
 #include "main/eventloop.h"
 #include "main/main.h"
-#include "main/md5.h"
 #include "main/rom.h"
 #include "main/savestates.h"
 #include "main/util.h"
 #include "main/version.h"
 #include "main/workqueue.h"
-#include "osd/screenshot.h"
+#include "main/screenshot.h"
+#include "main/netplay.h"
 #include "plugin/plugin.h"
 #include "vidext.h"
 
 /* some local state variables */
 static int l_CoreInit = 0;
 static int l_ROMOpen = 0;
+static int l_DiskOpen = 0;
+static int l_CallerUsingSDL = 0;
 
 /* functions exported outside of libmupen64plus to front-end application */
 EXPORT m64p_error CALL CoreStartup(int APIVersion, const char *ConfigPath, const char *DataPath, void *Context,
@@ -60,6 +63,9 @@ EXPORT m64p_error CALL CoreStartup(int APIVersion, const char *ConfigPath, const
     if (l_CoreInit)
         return M64ERR_ALREADY_INIT;
 
+    /* check wether the caller has already initialized SDL */
+    l_CallerUsingSDL = (SDL_WasInit(0) != 0);
+
     /* very first thing is to set the callback functions for debug info and state changing*/
     SetDebugCallback(DebugCallback, Context);
     SetStateCallback(StateCallback, Context2);
@@ -91,6 +97,12 @@ EXPORT m64p_error CALL CoreStartup(int APIVersion, const char *ConfigPath, const
     if (!main_set_core_defaults())
         return M64ERR_INTERNAL;
 
+    /* allocate base memory */
+    g_mem_base = init_mem_base();
+    if (g_mem_base == NULL) {
+        return M64ERR_NO_MEMORY;
+    }
+
     /* The ROM database contains MD5 hashes, goodnames, and some game-specific parameters */
     romdatabase_open();
 
@@ -111,8 +123,13 @@ EXPORT m64p_error CALL CoreShutdown(void)
     workqueue_shutdown();
     savestates_deinit();
 
-    /* tell SDL to shut down */
-    SDL_Quit();
+    /* if the calling code is using SDL, don't shut it down */
+    if (!l_CallerUsingSDL)
+        SDL_Quit();
+
+    /* deallocate base memory */
+    release_mem_base(g_mem_base);
+    g_mem_base = NULL;
 
     l_CoreInit = 0;
     return M64ERR_SUCCESS;
@@ -124,7 +141,7 @@ EXPORT m64p_error CALL CoreAttachPlugin(m64p_plugin_type PluginType, m64p_dynlib
 
     if (!l_CoreInit)
         return M64ERR_NOT_INIT;
-    if (g_EmulatorRunning || !l_ROMOpen)
+    if (g_EmulatorRunning || (!l_ROMOpen && !l_DiskOpen))
         return M64ERR_INVALID_STATE;
 
     rval = plugin_connect(PluginType, PluginLibHandle);
@@ -161,31 +178,58 @@ EXPORT m64p_error CALL CoreDoCommand(m64p_command Command, int ParamInt, void *P
         case M64CMD_NOP:
             return M64ERR_SUCCESS;
         case M64CMD_ROM_OPEN:
-            if (g_EmulatorRunning || l_ROMOpen)
+            if (g_EmulatorRunning || l_DiskOpen || l_ROMOpen)
                 return M64ERR_INVALID_STATE;
-            if (ParamPtr == NULL || ParamInt < 4096)
+            // ROM buffer size must be divisible by 4 to avoid out-of-bounds read in swap_copy_rom (v64/n64 formats)
+            if (ParamPtr == NULL || ParamInt < 4096 || ParamInt > CART_ROM_MAX_SIZE)
                 return M64ERR_INPUT_ASSERT;
             rval = open_rom((const unsigned char *) ParamPtr, ParamInt);
             if (rval == M64ERR_SUCCESS)
             {
                 l_ROMOpen = 1;
                 ScreenshotRomOpen();
-                cheat_init();
+                cheat_init(&g_cheat_ctx);
             }
             return rval;
         case M64CMD_ROM_CLOSE:
             if (g_EmulatorRunning || !l_ROMOpen)
                 return M64ERR_INVALID_STATE;
             l_ROMOpen = 0;
-            cheat_delete_all();
-            cheat_uninit();
+            cheat_delete_all(&g_cheat_ctx);
+            cheat_uninit(&g_cheat_ctx);
             return close_rom();
+        case M64CMD_DISK_OPEN:
+            if (g_EmulatorRunning || l_DiskOpen || l_ROMOpen)
+                return M64ERR_INVALID_STATE;
+            if (ParamPtr != NULL)
+                return M64ERR_INPUT_INVALID;
+            rval = open_disk();
+            if (rval == M64ERR_SUCCESS)
+            {
+                l_DiskOpen = 1;
+                ScreenshotRomOpen();
+                cheat_init(&g_cheat_ctx);
+            }
+            return rval;
+        case M64CMD_DISK_CLOSE:
+            if (g_EmulatorRunning || !l_DiskOpen)
+                return M64ERR_INVALID_STATE;
+            l_DiskOpen = 0;
+            cheat_delete_all(&g_cheat_ctx);
+            cheat_uninit(&g_cheat_ctx);
+            return close_disk();
+        case M64CMD_PIF_OPEN:
+            if (g_EmulatorRunning)
+                return M64ERR_INVALID_STATE;
+            if (ParamPtr == NULL || ParamInt < 1984 || ParamInt > 2048 || ParamInt % 4 != 0)
+                return M64ERR_INPUT_ASSERT;
+            return open_pif((const unsigned char *) ParamPtr, ParamInt);
         case M64CMD_ROM_GET_HEADER:
-            if (!l_ROMOpen)
+            if (!l_ROMOpen && !l_DiskOpen)
                 return M64ERR_INVALID_STATE;
             if (ParamPtr == NULL)
                 return M64ERR_INPUT_ASSERT;
-            if (sizeof(m64p_rom_header) < ParamInt)
+            if ((int)sizeof(m64p_rom_header) < ParamInt)
                 ParamInt = sizeof(m64p_rom_header);
             memcpy(ParamPtr, &ROM_HEADER, ParamInt);
             // Mupen64Plus used to keep a m64p_rom_header with a clean ROM name
@@ -197,16 +241,25 @@ EXPORT m64p_error CALL CoreDoCommand(m64p_command Command, int ParamInt, void *P
             }
             return M64ERR_SUCCESS;
         case M64CMD_ROM_GET_SETTINGS:
-            if (!l_ROMOpen)
+            if (!l_ROMOpen && !l_DiskOpen)
                 return M64ERR_INVALID_STATE;
             if (ParamPtr == NULL)
                 return M64ERR_INPUT_ASSERT;
-            if (sizeof(m64p_rom_settings) < ParamInt)
+            if ((int)sizeof(m64p_rom_settings) < ParamInt)
                 ParamInt = sizeof(m64p_rom_settings);
             memcpy(ParamPtr, &ROM_SETTINGS, ParamInt);
             return M64ERR_SUCCESS;
+        case M64CMD_ROM_SET_SETTINGS:
+            if (g_EmulatorRunning || (!l_ROMOpen && !l_DiskOpen))
+                return M64ERR_INVALID_STATE;
+            if (ParamPtr == NULL)
+                return M64ERR_INPUT_ASSERT;
+            if ((int)sizeof(m64p_rom_settings) < ParamInt)
+                ParamInt = sizeof(m64p_rom_settings);
+            memcpy(&ROM_SETTINGS, ParamPtr, ParamInt);
+            return M64ERR_SUCCESS;
         case M64CMD_EXECUTE:
-            if (g_EmulatorRunning || !l_ROMOpen)
+            if (g_EmulatorRunning || (!l_ROMOpen && !l_DiskOpen))
                 return M64ERR_INVALID_STATE;
             /* print out plugin-related warning messages */
             plugin_check();
@@ -235,8 +288,6 @@ EXPORT m64p_error CALL CoreDoCommand(m64p_command Command, int ParamInt, void *P
                 return M64ERR_INPUT_ASSERT;
             return main_core_state_set((m64p_core_param) ParamInt, *((int *)ParamPtr));
         case M64CMD_STATE_LOAD:
-            if (!g_EmulatorRunning)
-                return M64ERR_INVALID_STATE;
             main_state_load((char *) ParamPtr);
             return M64ERR_SUCCESS;
         case M64CMD_STATE_SAVE:
@@ -265,7 +316,7 @@ EXPORT m64p_error CALL CoreDoCommand(m64p_command Command, int ParamInt, void *P
             event_sdl_keyup(keysym, keymod);
             return M64ERR_SUCCESS;
         case M64CMD_SET_FRAME_CALLBACK:
-            g_FrameCallback = (m64p_frame_callback) ParamPtr;
+            *(void**)&g_FrameCallback = ParamPtr;
             return M64ERR_SUCCESS;
         case M64CMD_TAKE_NEXT_SCREENSHOT:
             if (!g_EmulatorRunning)
@@ -291,6 +342,35 @@ EXPORT m64p_error CALL CoreDoCommand(m64p_command Command, int ParamInt, void *P
                 return M64ERR_INVALID_STATE;
             main_advance_one();
             return M64ERR_SUCCESS;
+        case M64CMD_SET_MEDIA_LOADER:
+            if (ParamInt != sizeof(m64p_media_loader) || ParamPtr == NULL)
+                return M64ERR_INPUT_INVALID;
+            g_media_loader = *(m64p_media_loader*)ParamPtr;
+            return M64ERR_SUCCESS;
+        case M64CMD_NETPLAY_INIT:
+            if (ParamInt < 1 || ParamPtr == NULL)
+                return M64ERR_INPUT_INVALID;
+            return netplay_start(ParamPtr, ParamInt);
+        case M64CMD_NETPLAY_CONTROL_PLAYER:
+            if (ParamInt < 1 || ParamInt > 4 || ParamPtr == NULL)
+                return M64ERR_INPUT_INVALID;
+            if (netplay_register_player(ParamInt - 1, Controls[netplay_next_controller()].Plugin, Controls[netplay_next_controller()].RawData, *(uint32_t*)ParamPtr))
+            {
+                netplay_set_controller(ParamInt - 1);
+                return M64ERR_SUCCESS;
+            }
+            else
+                return M64ERR_INPUT_ASSERT; // player already in use
+        case M64CMD_NETPLAY_GET_VERSION:
+            if (ParamPtr == NULL)
+                return M64ERR_INPUT_INVALID;
+            *(uint32_t*)ParamPtr = NETPLAY_CORE_VERSION;
+            if (ParamInt == NETPLAY_API_VERSION)
+                return M64ERR_SUCCESS;
+            else
+                return M64ERR_INCOMPATIBLE;
+        case M64CMD_NETPLAY_CLOSE:
+            return netplay_stop();
         default:
             return M64ERR_INPUT_INVALID;
     }
@@ -310,12 +390,14 @@ EXPORT m64p_error CALL CoreAddCheat(const char *CheatName, m64p_cheat_code *Code
 {
     if (!l_CoreInit)
         return M64ERR_NOT_INIT;
+    if (netplay_is_init())
+        return M64ERR_INVALID_STATE;
     if (CheatName == NULL || CodeList == NULL)
         return M64ERR_INPUT_ASSERT;
     if (strlen(CheatName) < 1 || NumCodes < 1)
         return M64ERR_INPUT_INVALID;
 
-    if (cheat_add_new(CheatName, CodeList, NumCodes))
+    if (cheat_add_new(&g_cheat_ctx, CheatName, CodeList, NumCodes))
         return M64ERR_SUCCESS;
 
     return M64ERR_INPUT_INVALID;
@@ -325,10 +407,12 @@ EXPORT m64p_error CALL CoreCheatEnabled(const char *CheatName, int Enabled)
 {
     if (!l_CoreInit)
         return M64ERR_NOT_INIT;
+    if (netplay_is_init())
+        return M64ERR_INVALID_STATE;
     if (CheatName == NULL)
         return M64ERR_INPUT_ASSERT;
 
-    if (cheat_set_enabled(CheatName, Enabled))
+    if (cheat_set_enabled(&g_cheat_ctx, CheatName, Enabled))
         return M64ERR_SUCCESS;
 
     return M64ERR_INPUT_INVALID;
@@ -343,7 +427,7 @@ EXPORT m64p_error CALL CoreGetRomSettings(m64p_rom_settings *RomSettings, int Ro
         return M64ERR_NOT_INIT;
     if (RomSettings == NULL)
         return M64ERR_INPUT_ASSERT;
-    if (RomSettingsLength < sizeof(m64p_rom_settings))
+    if (RomSettingsLength < (int)sizeof(m64p_rom_settings))
         return M64ERR_INPUT_INVALID;
 
     /* Look up this ROM in the .ini file and fill in goodname, etc */
@@ -360,6 +444,13 @@ EXPORT m64p_error CALL CoreGetRomSettings(m64p_rom_settings *RomSettings, int Ro
     RomSettings->status = entry->status;
     RomSettings->players = entry->players;
     RomSettings->rumble = entry->rumble;
+    RomSettings->transferpak = entry->transferpak;
+    RomSettings->mempak = entry->mempak;
+    RomSettings->disableextramem = entry->disableextramem;
+    RomSettings->countperop = entry->countperop;
+    RomSettings->savetype = entry->savetype;
+    RomSettings->sidmaduration = entry->sidmaduration;
+    RomSettings->aidmamodifier = entry->aidmamodifier;
 
     return M64ERR_SUCCESS;
 }
