@@ -1,6 +1,6 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  *   Mupen64plus-core - api/vidext.c                                       *
- *   Mupen64Plus homepage: http://code.google.com/p/mupen64plus/           *
+ *   Mupen64Plus homepage: https://mupen64plus.org/                        *
  *   Copyright (C) 2009 Richard Goedeken                                   *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -24,10 +24,19 @@
  */
 
 #include <SDL.h>
+/* we need at least SDL 2.0.6 for vulkan */
+#if !SDL_VERSION_ATLEAST(2,0,6)
+    #undef VIDEXT_VULKAN
+#endif
+#ifdef VIDEXT_VULKAN
+#include <SDL_vulkan.h>
+#include <vulkan/vulkan.h>
+#endif
 #include <stdlib.h>
 #include <string.h>
 
 #define M64P_CORE_PROTOTYPES 1
+#include "osal/preproc.h"
 #include "../osd/osd.h"
 #include "callbacks.h"
 #include "m64p_types.h"
@@ -35,15 +44,23 @@
 #include "vidext.h"
 
 #if SDL_VERSION_ATLEAST(2,0,0)
+    #ifndef USE_GLES
+    static int l_ForceCompatibilityContext = 1;
+    #endif
 #include "vidext_sdl2_compat.h"
 #endif
 
 /* local variables */
-static m64p_video_extension_functions l_ExternalVideoFuncTable = {11, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL};
+static m64p_video_extension_functions l_ExternalVideoFuncTable = {17, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL};
 static int l_VideoExtensionActive = 0;
 static int l_VideoOutputActive = 0;
 static int l_Fullscreen = 0;
+static int l_SwapControl = 0;
+static m64p_render_mode l_RenderMode = M64P_RENDER_OPENGL;
 static SDL_Surface *l_pScreen = NULL;
+#ifdef VIDEXT_VULKAN
+static const char** l_VulkanExtensionNames = NULL;
+#endif
 
 /* global function for use by frontend.c */
 m64p_error OverrideVideoFunctions(m64p_video_extension_functions *VideoFunctionStruct)
@@ -51,24 +68,30 @@ m64p_error OverrideVideoFunctions(m64p_video_extension_functions *VideoFunctionS
     /* check input data */
     if (VideoFunctionStruct == NULL)
         return M64ERR_INPUT_ASSERT;
-    if (VideoFunctionStruct->Functions < 11)
+    if (VideoFunctionStruct->Functions < 17)
         return M64ERR_INPUT_INVALID;
 
     /* disable video extension if any of the function pointers are NULL */
     if (VideoFunctionStruct->VidExtFuncInit == NULL ||
+        VideoFunctionStruct->VidExtFuncInitWithRenderMode == NULL ||
         VideoFunctionStruct->VidExtFuncQuit == NULL ||
         VideoFunctionStruct->VidExtFuncListModes == NULL ||
+        VideoFunctionStruct->VidExtFuncListRates == NULL ||
         VideoFunctionStruct->VidExtFuncSetMode == NULL ||
+        VideoFunctionStruct->VidExtFuncSetModeWithRate == NULL ||
         VideoFunctionStruct->VidExtFuncGLGetProc == NULL ||
         VideoFunctionStruct->VidExtFuncGLSetAttr == NULL ||
         VideoFunctionStruct->VidExtFuncGLGetAttr == NULL ||
         VideoFunctionStruct->VidExtFuncGLSwapBuf == NULL ||
         VideoFunctionStruct->VidExtFuncSetCaption == NULL ||
         VideoFunctionStruct->VidExtFuncToggleFS == NULL ||
-        VideoFunctionStruct->VidExtFuncResizeWindow == NULL)
+        VideoFunctionStruct->VidExtFuncResizeWindow == NULL ||
+        VideoFunctionStruct->VidExtFuncGLGetDefaultFramebuffer == NULL ||
+        VideoFunctionStruct->VidExtFuncVKGetSurface == NULL ||
+        VideoFunctionStruct->VidExtFuncVKGetInstanceExtensions == NULL)
     {
-        l_ExternalVideoFuncTable.Functions = 11;
-        memset(&l_ExternalVideoFuncTable.VidExtFuncInit, 0, 11 * sizeof(void *));
+        l_ExternalVideoFuncTable.Functions = 17;
+        memset(&l_ExternalVideoFuncTable.VidExtFuncInit, 0, 17 * sizeof(void *));
         l_VideoExtensionActive = 0;
         return M64ERR_SUCCESS;
     }
@@ -96,12 +119,54 @@ EXPORT m64p_error CALL VidExt_Init(void)
     if (l_VideoExtensionActive)
         return (*l_ExternalVideoFuncTable.VidExtFuncInit)();
 
+    /* redirect to VidExt_InitWithRenderMode with OpenGL render mode */
+    return VidExt_InitWithRenderMode(M64P_RENDER_OPENGL);
+}
+
+EXPORT m64p_error CALL VidExt_InitWithRenderMode(m64p_render_mode RenderMode)
+{
+    /* call video extension override if necessary */
+    if (l_VideoExtensionActive)
+        return (*l_ExternalVideoFuncTable.VidExtFuncInitWithRenderMode)(RenderMode);
+
+    /* set global render mode */
+#ifdef VIDEXT_VULKAN
+    l_RenderMode = RenderMode;
+#endif
+
+#if SDL_VERSION_ATLEAST(2,0,0)
+    SDL_SetHint(SDL_HINT_JOYSTICK_ALLOW_BACKGROUND_EVENTS, "1");
+    /* retrieve default swap interval/VSync */
+    if (RenderMode == M64P_RENDER_OPENGL) {
+        l_SwapControl = SDL_GL_GetSwapInterval();
+    }
+#endif
+
+#if SDL_VERSION_ATLEAST(2,24,0)
+    /* fix DPI scaling issues on Windows */
+    SDL_SetHint(SDL_HINT_WINDOWS_DPI_AWARENESS, "permonitorv2");
+#endif
+
     if (SDL_InitSubSystem(SDL_INIT_VIDEO) == -1)
     {
         DebugMessage(M64MSG_ERROR, "SDL video subsystem init failed: %s", SDL_GetError());
         return M64ERR_SYSTEM_FAIL;
     }
 
+    /* attempt to load vulkan library  */
+    if (RenderMode == M64P_RENDER_VULKAN)
+    {
+#ifdef VIDEXT_VULKAN
+        if (SDL_Vulkan_LoadLibrary(NULL) == -1)
+        {
+            DebugMessage(M64MSG_ERROR, "SDL_Vulkan_LoadLibrary failed: %s", SDL_GetError());
+            return M64ERR_SYSTEM_FAIL;
+        }
+#else
+        return M64ERR_UNSUPPORTED;
+#endif
+    }
+
     return M64ERR_SUCCESS;
 }
 
@@ -125,6 +190,15 @@ EXPORT m64p_error CALL VidExt_Quit(void)
     SDL_ShowCursor(SDL_ENABLE);
 #if SDL_VERSION_ATLEAST(2,0,0)
     SDL2_DestroyWindow();
+#endif
+#ifdef VIDEXT_VULKAN
+    if (l_RenderMode == M64P_RENDER_VULKAN) {
+        SDL_Vulkan_UnloadLibrary();
+    }
+    if (l_VulkanExtensionNames != NULL) {
+        free(l_VulkanExtensionNames);
+        l_VulkanExtensionNames = NULL;
+    }
 #endif
     SDL_QuitSubSystem(SDL_INIT_VIDEO);
     l_pScreen = NULL;
@@ -184,6 +258,53 @@ EXPORT m64p_error CALL VidExt_ListFullscreenModes(m64p_2d_size *SizeArray, int *
     return M64ERR_SUCCESS;
 }
 
+EXPORT m64p_error CALL VidExt_ListFullscreenRates(m64p_2d_size Size, int *NumRates, int *Rates)
+{
+    /* call video extension override if necessary */
+    if (l_VideoExtensionActive)
+        return (*l_ExternalVideoFuncTable.VidExtFuncListRates)(Size, NumRates, Rates);
+
+#if SDL_VERSION_ATLEAST(2,0,0)
+    if (!SDL_WasInit(SDL_INIT_VIDEO))
+        return M64ERR_NOT_INIT;
+
+    int display = GetVideoDisplay();
+    int modeCount = SDL_GetNumDisplayModes(display);
+    SDL_DisplayMode displayMode;
+
+    if (modeCount < 1)
+    {
+        DebugMessage(M64MSG_ERROR, "SDL_GetNumDisplayModes failed: %s", SDL_GetError());
+        return M64ERR_SYSTEM_FAIL;
+    }
+
+    int rateCount = 0;
+    for (int i = 0; (i < modeCount) && (rateCount < *NumRates); i++)
+    {
+        if (SDL_GetDisplayMode(display, i, &displayMode) < 0)
+        {
+            DebugMessage(M64MSG_ERROR, "SDL_GetDisplayMode failed: %s", SDL_GetError());
+            return M64ERR_SYSTEM_FAIL;
+        }
+
+        /* skip when we're not at the right resolution */
+        if (displayMode.w != (int)Size.uiWidth ||
+            displayMode.h != (int)Size.uiHeight)
+            continue;
+
+        Rates[rateCount] = displayMode.refresh_rate;
+        rateCount++;
+    }
+
+    *NumRates = rateCount;
+
+    return M64ERR_SUCCESS;
+#else
+    // SDL1 doesn't support getting refresh rates
+    return M64ERR_UNSUPPORTED;
+#endif
+}
+
 EXPORT m64p_error CALL VidExt_SetVideoMode(int Width, int Height, int BitsPerPixel, m64p_video_mode ScreenMode, m64p_video_flags Flags)
 {
     const SDL_VideoInfo *videoInfo;
@@ -207,15 +328,24 @@ EXPORT m64p_error CALL VidExt_SetVideoMode(int Width, int Height, int BitsPerPix
         return M64ERR_NOT_INIT;
 
     /* Get SDL video flags to use */
-    if (ScreenMode == M64VIDEO_WINDOWED)
+    if (l_RenderMode == M64P_RENDER_OPENGL)
     {
         videoFlags = SDL_OPENGL;
+    }
+#ifdef VIDEXT_VULKAN
+    else
+    {
+        videoFlags = SDL_VULKAN;
+    }
+#endif
+    if (ScreenMode == M64VIDEO_WINDOWED)
+    {
         if (Flags & M64VIDEOFLAG_SUPPORT_RESIZING)
             videoFlags |= SDL_RESIZABLE;
     }
     else if (ScreenMode == M64VIDEO_FULLSCREEN)
     {
-        videoFlags = SDL_OPENGL | SDL_FULLSCREEN;
+        videoFlags |= SDL_FULLSCREEN;
     }
     else
     {
@@ -247,6 +377,15 @@ EXPORT m64p_error CALL VidExt_SetVideoMode(int Width, int Height, int BitsPerPix
 
     SDL_ShowCursor(SDL_DISABLE);
 
+#if SDL_VERSION_ATLEAST(2,0,0)
+    /* set swap interval/VSync */
+    if (l_RenderMode == M64P_RENDER_OPENGL &&
+        SDL_GL_SetSwapInterval(l_SwapControl) != 0)
+    {
+        DebugMessage(M64MSG_ERROR, "SDL swap interval (VSync) set failed: %s", SDL_GetError());
+    }
+#endif
+
     l_Fullscreen = (ScreenMode == M64VIDEO_FULLSCREEN);
     l_VideoOutputActive = 1;
     StateChanged(M64CORE_VIDEO_MODE, ScreenMode);
@@ -254,6 +393,105 @@ EXPORT m64p_error CALL VidExt_SetVideoMode(int Width, int Height, int BitsPerPix
     return M64ERR_SUCCESS;
 }
 
+EXPORT m64p_error CALL VidExt_SetVideoModeWithRate(int Width, int Height, int RefreshRate, int BitsPerPixel, m64p_video_mode ScreenMode, m64p_video_flags Flags)
+{
+    /* call video extension override if necessary */
+    if (l_VideoExtensionActive)
+    {
+        m64p_error rval = (*l_ExternalVideoFuncTable.VidExtFuncSetModeWithRate)(Width, Height, RefreshRate, BitsPerPixel, ScreenMode, Flags);
+        l_Fullscreen = (rval == M64ERR_SUCCESS && ScreenMode == M64VIDEO_FULLSCREEN);
+        l_VideoOutputActive = (rval == M64ERR_SUCCESS);
+        if (l_VideoOutputActive)
+        {
+            StateChanged(M64CORE_VIDEO_MODE, ScreenMode);
+            StateChanged(M64CORE_VIDEO_SIZE, (Width << 16) | Height);
+        }
+        return rval;
+    }
+
+#if SDL_VERSION_ATLEAST(2,0,0)
+    if (!SDL_WasInit(SDL_INIT_VIDEO) || !SDL_VideoWindow)
+        return M64ERR_NOT_INIT;
+
+    int videoFlags = 0;
+    int display = GetVideoDisplay();
+    int modeCount = SDL_GetNumDisplayModes(display);
+    SDL_DisplayMode displayMode;
+    int modeFound = 0;
+
+    /* Get SDL video flags to use */
+    if (ScreenMode == M64VIDEO_WINDOWED)
+        videoFlags = 0;
+    else if (ScreenMode == M64VIDEO_FULLSCREEN)
+        videoFlags = SDL_WINDOW_FULLSCREEN;
+    else
+        return M64ERR_INPUT_INVALID;
+
+    if (modeCount < 1)
+    {
+        DebugMessage(M64MSG_ERROR, "SDL_GetNumDisplayModes failed: %s", SDL_GetError());
+        return M64ERR_SYSTEM_FAIL;
+    }
+
+    /* Attempt to find valid screen mode */
+    for (int i = 0; i < modeCount; i++)
+    {
+        if (SDL_GetDisplayMode(display, i, &displayMode) < 0)
+        {
+            DebugMessage(M64MSG_ERROR, "SDL_GetDisplayMode failed: %s", SDL_GetError());
+            return M64ERR_SYSTEM_FAIL;
+        }
+
+        /* skip when we're not at the right mode */
+        if (displayMode.w != Width ||
+            displayMode.h != Height ||
+            displayMode.refresh_rate != RefreshRate)
+            continue;
+
+        modeFound = 1;
+        break;
+    }
+
+    /* return when no modes with specifed size have been found */
+    if (modeFound == 0)
+        return M64ERR_INPUT_INVALID;
+
+    /* Set window in specified mode */
+    if (SDL_SetWindowFullscreen(SDL_VideoWindow, videoFlags) < 0)
+    {
+        DebugMessage(M64MSG_ERROR, "SDL_SetWindowFullscreen failed: %s", SDL_GetError());
+        return M64ERR_SYSTEM_FAIL;
+    }
+
+    if (ScreenMode == M64VIDEO_FULLSCREEN)
+    {
+        if (SDL_SetWindowDisplayMode(SDL_VideoWindow, &displayMode) < 0)
+        {
+            DebugMessage(M64MSG_ERROR, "SDL_SetWindowDisplayMode failed: %s", SDL_GetError());
+            return M64ERR_SYSTEM_FAIL;
+        }
+    }
+
+    SDL_ShowCursor(SDL_DISABLE);
+
+    /* set swap interval/VSync */
+    if (SDL_GL_SetSwapInterval(l_SwapControl) != 0)
+    {
+        DebugMessage(M64MSG_ERROR, "SDL swap interval (VSync) set failed: %s", SDL_GetError());
+    }
+
+    l_Fullscreen = (ScreenMode == M64VIDEO_FULLSCREEN);
+    l_VideoOutputActive = 1;
+    StateChanged(M64CORE_VIDEO_MODE, ScreenMode);
+    StateChanged(M64CORE_VIDEO_SIZE, (Width << 16) | Height);
+
+    return M64ERR_SUCCESS;
+#else
+    // SDL1 doesn't support setting refresh rates
+    return M64ERR_UNSUPPORTED;
+#endif
+}
+
 EXPORT m64p_error CALL VidExt_ResizeWindow(int Width, int Height)
 {
     const SDL_VideoInfo *videoInfo;
@@ -286,7 +524,13 @@ EXPORT m64p_error CALL VidExt_ResizeWindow(int Width, int Height)
     }
 
     /* Get SDL video flags to use */
-    videoFlags = SDL_OPENGL | SDL_RESIZABLE;
+    if (l_RenderMode == M64P_RENDER_OPENGL)
+        videoFlags = SDL_OPENGL;
+#ifdef VIDEXT_VULKAN
+    else
+        videoFlags = SDL_VULKAN;
+#endif
+    videoFlags |= SDL_RESIZABLE;
     if ((videoInfo = SDL_GetVideoInfo()) == NULL)
     {
         DebugMessage(M64MSG_ERROR, "SDL_GetVideoInfo query failed: %s", SDL_GetError());
@@ -359,20 +603,31 @@ EXPORT m64p_error CALL VidExt_ToggleFullScreen(void)
         StateChanged(M64CORE_VIDEO_MODE, l_Fullscreen ? M64VIDEO_FULLSCREEN : M64VIDEO_WINDOWED);
         return M64ERR_SUCCESS;
     }
+    else
+    {
+        DebugMessage(M64MSG_ERROR, "SDL_WM_ToggleFullScreen failed: %s", SDL_GetError());
+    }
 
     return M64ERR_SYSTEM_FAIL;
 }
 
-EXPORT void * CALL VidExt_GL_GetProcAddress(const char* Proc)
+EXPORT m64p_function CALL VidExt_GL_GetProcAddress(const char* Proc)
 {
     /* call video extension override if necessary */
     if (l_VideoExtensionActive)
         return (*l_ExternalVideoFuncTable.VidExtFuncGLGetProc)(Proc);
 
+    if (l_RenderMode != M64P_RENDER_OPENGL)
+        return NULL;
+
     if (!SDL_WasInit(SDL_INIT_VIDEO))
         return NULL;
 
-    return SDL_GL_GetProcAddress(Proc);
+/* WARN: assume cast to m64p_function is supported by platform and disable warning accordingly */
+OSAL_WARNING_PUSH
+OSAL_NO_WARNING_FPTR_VOIDP_CAST
+    return (m64p_function)SDL_GL_GetProcAddress(Proc);
+OSAL_WARNING_POP
 }
 
 typedef struct {
@@ -388,9 +643,7 @@ static const GLAttrMapNode GLAttrMap[] = {
         { M64P_GL_GREEN_SIZE,   SDL_GL_GREEN_SIZE },
         { M64P_GL_BLUE_SIZE,    SDL_GL_BLUE_SIZE },
         { M64P_GL_ALPHA_SIZE,   SDL_GL_ALPHA_SIZE },
-#if SDL_VERSION_ATLEAST(1,3,0)
-        { M64P_GL_SWAP_CONTROL, SDL_RENDERER_PRESENTVSYNC },
-#else
+#if !SDL_VERSION_ATLEAST(1,3,0)
         { M64P_GL_SWAP_CONTROL, SDL_GL_SWAP_CONTROL },
 #endif
         { M64P_GL_MULTISAMPLEBUFFERS, SDL_GL_MULTISAMPLEBUFFERS },
@@ -411,9 +664,18 @@ EXPORT m64p_error CALL VidExt_GL_SetAttribute(m64p_GLattr Attr, int Value)
     if (l_VideoExtensionActive)
         return (*l_ExternalVideoFuncTable.VidExtFuncGLSetAttr)(Attr, Value);
 
+    if (l_RenderMode != M64P_RENDER_OPENGL)
+        return M64ERR_INVALID_STATE;
+
     if (!SDL_WasInit(SDL_INIT_VIDEO))
         return M64ERR_NOT_INIT;
 
+    if (Attr == M64P_GL_SWAP_CONTROL)
+    {
+        /* SDL swap interval/vsync needs to be set on current GL context, so save it for later */
+        l_SwapControl = Value;
+    }
+
     /* translate the GL context type mask if necessary */
 #if SDL_VERSION_ATLEAST(2,0,0)
     if (Attr == M64P_GL_CONTEXT_PROFILE_MASK)
@@ -422,6 +684,9 @@ EXPORT m64p_error CALL VidExt_GL_SetAttribute(m64p_GLattr Attr, int Value)
         {
             case M64P_GL_CONTEXT_PROFILE_CORE:
                 Value = SDL_GL_CONTEXT_PROFILE_CORE;
+#ifndef USE_GLES
+                l_ForceCompatibilityContext = 0;
+#endif
                 break;
             case M64P_GL_CONTEXT_PROFILE_COMPATIBILITY:
                 Value = SDL_GL_CONTEXT_PROFILE_COMPATIBILITY;
@@ -456,9 +721,20 @@ EXPORT m64p_error CALL VidExt_GL_GetAttribute(m64p_GLattr Attr, int *pValue)
     if (l_VideoExtensionActive)
         return (*l_ExternalVideoFuncTable.VidExtFuncGLGetAttr)(Attr, pValue);
 
+    if (l_RenderMode != M64P_RENDER_OPENGL)
+        return M64ERR_INVALID_STATE;
+
     if (!SDL_WasInit(SDL_INIT_VIDEO))
         return M64ERR_NOT_INIT;
 
+#if SDL_VERSION_ATLEAST(2,0,0)
+    if (Attr == M64P_GL_SWAP_CONTROL)
+    {
+        *pValue = SDL_GL_GetSwapInterval();
+        return M64ERR_SUCCESS;
+    }
+#endif
+
     for (i = 0; i < mapSize; i++)
     {
         if (GLAttrMap[i].m64Attr == Attr)
@@ -500,6 +776,9 @@ EXPORT m64p_error CALL VidExt_GL_SwapBuffers(void)
     if (l_VideoExtensionActive)
         return (*l_ExternalVideoFuncTable.VidExtFuncGLSwapBuf)();
 
+    if (l_RenderMode != M64P_RENDER_OPENGL)
+        return M64ERR_INVALID_STATE;
+
     if (!SDL_WasInit(SDL_INIT_VIDEO))
         return M64ERR_NOT_INIT;
 
@@ -507,4 +786,79 @@ EXPORT m64p_error CALL VidExt_GL_SwapBuffers(void)
     return M64ERR_SUCCESS;
 }
 
+EXPORT uint32_t CALL VidExt_GL_GetDefaultFramebuffer(void)
+{
+    if (l_VideoExtensionActive)
+        return (*l_ExternalVideoFuncTable.VidExtFuncGLGetDefaultFramebuffer)();
 
+    return 0;
+}
+
+EXPORT m64p_error CALL VidExt_VK_GetSurface(void** Surface, void* Instance)
+{
+    if (l_VideoExtensionActive)
+        return (*l_ExternalVideoFuncTable.VidExtFuncVKGetSurface)(Surface, Instance);
+
+#ifdef VIDEXT_VULKAN
+    VkSurfaceKHR vulkanSurface = VK_NULL_HANDLE;
+
+    if (l_RenderMode != M64P_RENDER_VULKAN)
+        return M64ERR_INVALID_STATE;
+
+    if (!SDL_WasInit(SDL_INIT_VIDEO) || !SDL_VideoWindow)
+        return M64ERR_NOT_INIT;
+
+    if (SDL_Vulkan_CreateSurface(SDL_VideoWindow, (VkInstance)Instance, &vulkanSurface) == SDL_FALSE) {
+        DebugMessage(M64MSG_ERROR, "SDL_Vulkan_CreateSurface failed: %s", SDL_GetError());
+        return M64ERR_SYSTEM_FAIL;
+    }
+
+    *Surface = (void*)vulkanSurface;
+    return M64ERR_SUCCESS;
+#else
+    return M64ERR_UNSUPPORTED;
+#endif
+}
+
+EXPORT m64p_error CALL VidExt_VK_GetInstanceExtensions(const char** Extensions[], uint32_t* NumExtensions)
+{
+    if (l_VideoExtensionActive)
+        return (*l_ExternalVideoFuncTable.VidExtFuncVKGetInstanceExtensions)(Extensions, NumExtensions);
+
+#ifdef VIDEXT_VULKAN
+    if (l_RenderMode != M64P_RENDER_VULKAN)
+        return M64ERR_INVALID_STATE;
+
+    if (!SDL_WasInit(SDL_INIT_VIDEO))
+        return M64ERR_NOT_INIT;
+
+    unsigned int extensionCount = 0;
+    if (SDL_Vulkan_GetInstanceExtensions(NULL, &extensionCount, NULL) == SDL_FALSE) {
+        DebugMessage(M64MSG_ERROR, "SDL_Vulkan_GetInstanceExtensions failed: %s", SDL_GetError());
+        return M64ERR_SYSTEM_FAIL;
+    }
+
+    /* ensure names have been freed before allocating it again */
+    if (l_VulkanExtensionNames != NULL) {
+        free(l_VulkanExtensionNames);
+        l_VulkanExtensionNames = NULL;
+    }
+
+    l_VulkanExtensionNames = malloc(sizeof(const char*) * extensionCount);
+    if (l_VulkanExtensionNames == NULL) {
+        DebugMessage(M64MSG_ERROR, "malloc failed");
+        return M64ERR_SYSTEM_FAIL;
+    }
+
+    if (SDL_Vulkan_GetInstanceExtensions(NULL, &extensionCount, l_VulkanExtensionNames) == SDL_FALSE) {
+        DebugMessage(M64MSG_ERROR, "SDL_Vulkan_GetInstanceExtensions failed: %s", SDL_GetError());
+        return M64ERR_SYSTEM_FAIL;
+    }
+
+    *NumExtensions = extensionCount;
+    *Extensions    = l_VulkanExtensionNames;
+    return M64ERR_SUCCESS;
+#else
+    return M64ERR_UNSUPPORTED;
+#endif
+}
