@@ -1,6 +1,6 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  *   Mupen64plus - rom.c                                                   *
- *   Mupen64Plus homepage: http://code.google.com/p/mupen64plus/           *
+ *   Mupen64Plus homepage: https://mupen64plus.org/                        *
  *   Copyright (C) 2008 Tillin9                                            *
  *   Copyright (C) 2002 Hacktarux                                          *
  *                                                                         *
@@ -35,48 +35,53 @@
 #include "api/config.h"
 #include "api/m64p_config.h"
 #include "api/m64p_types.h"
+#include "device/dd/disk.h"
+#include "backends/file_storage.h"
+#include "device/device.h"
 #include "main.h"
 #include "md5.h"
-#include "memory/memory.h"
+#include "osal/files.h"
 #include "osal/preproc.h"
 #include "osd/osd.h"
-#include "r4300/r4300.h"
 #include "rom.h"
 #include "util.h"
 
-#define DEFAULT 16
-
 #define CHUNKSIZE 1024*128 /* Read files 128KB at a time. */
 
+/* Number of cpu cycles per instruction */
+enum { DEFAULT_COUNT_PER_OP = 2 };
+/* by default, extra mem is enabled */
+enum { DEFAULT_DISABLE_EXTRA_MEM = 0 };
+/* Default SI DMA duration */
+enum { DEFAULT_SI_DMA_DURATION = 0x900 };
+/* Default AI DMA modifier */
+enum { DEFAULT_AI_DMA_MODIFIER = 100 };
+
 static romdatabase_entry* ini_search_by_md5(md5_byte_t* md5);
 
 static _romdatabase g_romdatabase;
 
-/* Global loaded rom memory space. */
-unsigned char* g_rom = NULL;
 /* Global loaded rom size. */
 int g_rom_size = 0;
 
-unsigned char isGoldeneyeRom = 0;
-
 m64p_rom_header   ROM_HEADER;
 rom_params        ROM_PARAMS;
 m64p_rom_settings ROM_SETTINGS;
 
 static m64p_system_type rom_country_code_to_system_type(uint16_t country_code);
-static int rom_system_type_to_ai_dac_rate(m64p_system_type system_type);
-static int rom_system_type_to_vi_limit(m64p_system_type system_type);
+
+static unsigned char rom_homebrew_savetype_to_savetype(uint8_t save_type);
 
 static const uint8_t Z64_SIGNATURE[4] = { 0x80, 0x37, 0x12, 0x40 };
 static const uint8_t V64_SIGNATURE[4] = { 0x37, 0x80, 0x40, 0x12 };
 static const uint8_t N64_SIGNATURE[4] = { 0x40, 0x12, 0x37, 0x80 };
 
-/* Tests if a file is a valid N64 rom by checking the first 4 bytes. */
-static int is_valid_rom(const unsigned char *buffer)
+/* Tests if a file is a valid N64 rom by checking the first 4 bytes and size */
+static int is_valid_rom(const unsigned char *buffer, unsigned int size)
 {
-    if (memcmp(buffer, Z64_SIGNATURE, sizeof(Z64_SIGNATURE)) == 0
-     || memcmp(buffer, V64_SIGNATURE, sizeof(V64_SIGNATURE)) == 0
-     || memcmp(buffer, N64_SIGNATURE, sizeof(N64_SIGNATURE)) == 0)
+    if ((memcmp(buffer, Z64_SIGNATURE, sizeof(Z64_SIGNATURE)) == 0)
+     || (memcmp(buffer, V64_SIGNATURE, sizeof(V64_SIGNATURE)) == 0 && size % 2 == 0)
+     || (memcmp(buffer, N64_SIGNATURE, sizeof(N64_SIGNATURE)) == 0 && size % 4 == 0))
         return 1;
     else
         return 0;
@@ -141,31 +146,24 @@ m64p_error open_rom(const unsigned char* romimage, unsigned int size)
     int i;
 
     /* check input requirements */
-    if (g_rom != NULL)
-    {
-        DebugMessage(M64MSG_ERROR, "open_rom(): previous ROM image was not freed");
-        return M64ERR_INTERNAL;
-    }
-    if (romimage == NULL || !is_valid_rom(romimage))
+    if (romimage == NULL || !is_valid_rom(romimage, size))
     {
         DebugMessage(M64MSG_ERROR, "open_rom(): not a valid ROM image");
         return M64ERR_INPUT_INVALID;
     }
 
     /* Clear Byte-swapped flag, since ROM is now deleted. */
-    g_MemHasBeenBSwapped = 0;
+    g_RomWordsLittleEndian = 0;
     /* allocate new buffer for ROM and copy into this buffer */
     g_rom_size = size;
-    g_rom = (unsigned char *) malloc(size);
-    if (g_rom == NULL)
-        return M64ERR_NO_MEMORY;
-    swap_copy_rom(g_rom, romimage, size, &imagetype);
+    swap_copy_rom((uint8_t*)mem_base_u32(g_mem_base, MM_CART_ROM), romimage, size, &imagetype);
+    /* ROM is now in N64 native (big endian) byte order */
 
-    memcpy(&ROM_HEADER, g_rom, sizeof(m64p_rom_header));
+    memcpy(&ROM_HEADER, (uint8_t*)mem_base_u32(g_mem_base, MM_CART_ROM), sizeof(m64p_rom_header));
 
     /* Calculate MD5 hash  */
     md5_init(&state);
-    md5_append(&state, (const md5_byte_t*)g_rom, g_rom_size);
+    md5_append(&state, (const md5_byte_t*)((uint8_t*)mem_base_u32(g_mem_base, MM_CART_ROM)), g_rom_size);
     md5_finish(&state, digest);
     for ( i = 0; i < 16; ++i )
         sprintf(buffer+i*2, "%02X", digest[i]);
@@ -174,9 +172,6 @@ m64p_error open_rom(const unsigned char* romimage, unsigned int size)
 
     /* add some useful properties to ROM_PARAMS */
     ROM_PARAMS.systemtype = rom_country_code_to_system_type(ROM_HEADER.Country_code);
-    ROM_PARAMS.vilimit = rom_system_type_to_vi_limit(ROM_PARAMS.systemtype);
-    ROM_PARAMS.aidacrate = rom_system_type_to_ai_dac_rate(ROM_PARAMS.systemtype);
-    ROM_PARAMS.countperop = COUNT_PER_OP_DEFAULT;
     ROM_PARAMS.cheats = NULL;
 
     memcpy(ROM_PARAMS.headername, ROM_HEADER.Name, 20);
@@ -185,7 +180,7 @@ m64p_error open_rom(const unsigned char* romimage, unsigned int size)
 
     /* Look up this ROM in the .ini file and fill in goodname, etc */
     if ((entry=ini_search_by_md5(digest)) != NULL ||
-        (entry=ini_search_by_crc(sl(ROM_HEADER.CRC1),sl(ROM_HEADER.CRC2))) != NULL)
+        (entry=ini_search_by_crc(tohl(ROM_HEADER.CRC1),tohl(ROM_HEADER.CRC2))) != NULL)
     {
         strncpy(ROM_SETTINGS.goodname, entry->goodname, 255);
         ROM_SETTINGS.goodname[255] = '\0';
@@ -193,19 +188,42 @@ m64p_error open_rom(const unsigned char* romimage, unsigned int size)
         ROM_SETTINGS.status = entry->status;
         ROM_SETTINGS.players = entry->players;
         ROM_SETTINGS.rumble = entry->rumble;
-        ROM_PARAMS.countperop = entry->countperop;
+        ROM_SETTINGS.transferpak = entry->transferpak;
+        ROM_SETTINGS.mempak = entry->mempak;
+        ROM_SETTINGS.biopak = entry->biopak;
+        ROM_SETTINGS.countperop = entry->countperop;
+        ROM_SETTINGS.disableextramem = entry->disableextramem;
+        ROM_SETTINGS.sidmaduration = entry->sidmaduration;
+        ROM_SETTINGS.aidmamodifier = entry->aidmamodifier;
         ROM_PARAMS.cheats = entry->cheats;
     }
     else
     {
         strcpy(ROM_SETTINGS.goodname, ROM_PARAMS.headername);
         strcat(ROM_SETTINGS.goodname, " (unknown rom)");
-        ROM_SETTINGS.savetype = NONE;
         ROM_SETTINGS.status = 0;
-        ROM_SETTINGS.players = 0;
-        ROM_SETTINGS.rumble = 0;
-        ROM_PARAMS.countperop = COUNT_PER_OP_DEFAULT;
+        ROM_SETTINGS.players = 4;
+        ROM_SETTINGS.rumble = 1;
+        ROM_SETTINGS.transferpak = 0;
+        ROM_SETTINGS.mempak = 1;
+        ROM_SETTINGS.biopak = 0;
+        ROM_SETTINGS.countperop = DEFAULT_COUNT_PER_OP;
+        ROM_SETTINGS.disableextramem = DEFAULT_DISABLE_EXTRA_MEM;
+        ROM_SETTINGS.sidmaduration = DEFAULT_SI_DMA_DURATION;
+        ROM_SETTINGS.aidmamodifier = DEFAULT_AI_DMA_MODIFIER;
         ROM_PARAMS.cheats = NULL;
+
+        /* check if ROM has the Advanced Homebrew ROM Header (see https://n64brew.dev/wiki/ROM_Header) */
+        if (ROM_HEADER.Cartridge_ID == 0x4445)
+        {
+            /* When current ROM has the Advanced Homebrew ROM Header, use the save type */
+            ROM_SETTINGS.savetype = rom_homebrew_savetype_to_savetype(ROM_HEADER.Version >> 4);
+        }
+        else
+        {
+            /* There's no way to guess the save type, but 4K EEPROM is better than nothing */
+            ROM_SETTINGS.savetype = SAVETYPE_EEPROM_4K;
+        }
     }
 
     /* print out a bunch of info about the ROM */
@@ -213,51 +231,166 @@ m64p_error open_rom(const unsigned char* romimage, unsigned int size)
     DebugMessage(M64MSG_INFO, "Name: %s", ROM_HEADER.Name);
     imagestring(imagetype, buffer);
     DebugMessage(M64MSG_INFO, "MD5: %s", ROM_SETTINGS.MD5);
-    DebugMessage(M64MSG_INFO, "CRC: %" PRIX32 " %" PRIX32, sl(ROM_HEADER.CRC1), sl(ROM_HEADER.CRC2));
+    DebugMessage(M64MSG_INFO, "CRC: %08" PRIX32 " %08" PRIX32, tohl(ROM_HEADER.CRC1), tohl(ROM_HEADER.CRC2));
     DebugMessage(M64MSG_INFO, "Imagetype: %s", buffer);
     DebugMessage(M64MSG_INFO, "Rom size: %d bytes (or %d Mb or %d Megabits)", g_rom_size, g_rom_size/1024/1024, g_rom_size/1024/1024*8);
-    DebugMessage(M64MSG_VERBOSE, "ClockRate = %" PRIX32, sl(ROM_HEADER.ClockRate));
-    DebugMessage(M64MSG_INFO, "Version: %" PRIX32, sl(ROM_HEADER.Release));
-    if(sl(ROM_HEADER.Manufacturer_ID) == 'N')
+    DebugMessage(M64MSG_VERBOSE, "ClockRate = %" PRIX32, tohl(ROM_HEADER.ClockRate));
+    DebugMessage(M64MSG_INFO, "Version: %" PRIX32, tohl(ROM_HEADER.Release));
+    if(tohl(ROM_HEADER.Manufacturer_ID) == 'N')
         DebugMessage(M64MSG_INFO, "Manufacturer: Nintendo");
     else
-        DebugMessage(M64MSG_INFO, "Manufacturer: %" PRIX32, sl(ROM_HEADER.Manufacturer_ID));
+        DebugMessage(M64MSG_INFO, "Manufacturer: %" PRIX32, tohl(ROM_HEADER.Manufacturer_ID));
     DebugMessage(M64MSG_VERBOSE, "Cartridge_ID: %" PRIX16, ROM_HEADER.Cartridge_ID);
     countrycodestring(ROM_HEADER.Country_code, buffer);
     DebugMessage(M64MSG_INFO, "Country: %s", buffer);
-    DebugMessage(M64MSG_VERBOSE, "PC = %" PRIX32, sl(ROM_HEADER.PC));
+    DebugMessage(M64MSG_VERBOSE, "PC = %" PRIX32, tohl(ROM_HEADER.PC));
     DebugMessage(M64MSG_VERBOSE, "Save type: %d", ROM_SETTINGS.savetype);
 
-    //Prepare Hack for GOLDENEYE
-    isGoldeneyeRom = 0;
-    if(strcmp(ROM_PARAMS.headername, "GOLDENEYE") == 0)
-       isGoldeneyeRom = 1;
-
     return M64ERR_SUCCESS;
 }
 
 m64p_error close_rom(void)
 {
-    if (g_rom == NULL)
-        return M64ERR_INVALID_STATE;
-
-    free(g_rom);
-    g_rom = NULL;
-
     /* Clear Byte-swapped flag, since ROM is now deleted. */
-    g_MemHasBeenBSwapped = 0;
+    g_RomWordsLittleEndian = 0;
     DebugMessage(M64MSG_STATUS, "Rom closed.");
 
     return M64ERR_SUCCESS;
 }
 
+m64p_error open_disk(void)
+{
+    md5_state_t state;
+    md5_byte_t digest[16];
+    romdatabase_entry* entry;
+    char buffer[256];
+    int i;
+
+    /* ask the core loader for DD disk filename */
+    char* dd_disk_filename = (g_media_loader.get_dd_disk == NULL)
+        ? NULL
+        : g_media_loader.get_dd_disk(g_media_loader.cb_data);
+
+    /* handle the no disk case */
+    if (dd_disk_filename == NULL || strlen(dd_disk_filename) == 0) {
+        goto no_disk;
+    }
+
+    /* Get DD Disk size */
+    size_t dd_size = 0;
+    if (get_file_size(dd_disk_filename, &dd_size) != file_ok) {
+        goto no_disk;
+    }
+
+    struct file_storage* fstorage = malloc(sizeof(struct file_storage));
+    if (fstorage == NULL) {
+        goto no_disk;
+    }
+
+    /* Try loading regular disk file */
+    if (open_rom_file_storage(fstorage, dd_disk_filename) != file_ok) {
+        goto free_fstorage;
+    }
+
+    /* Scan disk to deduce disk format and other parameters and expand its size for D64 */
+    unsigned int format = 0;
+    unsigned int development = 0;
+    size_t offset_sys = 0;
+    size_t offset_id = 0;
+    size_t offset_ram = 0;
+    size_t size_ram = 0;
+    uint8_t* new_data = scan_and_expand_disk_format(fstorage->data, fstorage->size, &format, &development, &offset_sys, &offset_id, &offset_ram, &size_ram);
+    if (new_data == NULL) {
+        goto wrong_disk_format;
+    }
+    else {
+        fstorage->data = new_data;
+    }
+
+    /* Calculate MD5 hash  */
+    md5_init(&state);
+    md5_append(&state, (const md5_byte_t*)(fstorage->data), fstorage->size);
+    md5_finish(&state, digest);
+    for ( i = 0; i < 16; ++i )
+        sprintf(buffer+i*2, "%02X", digest[i]);
+    buffer[32] = '\0';
+    strcpy(ROM_SETTINGS.MD5, buffer);
+
+    /* Look up this disk in the .ini file and fill in goodname, etc */
+    if ((entry=ini_search_by_md5(digest)) != NULL)
+    {
+        strncpy(ROM_SETTINGS.goodname, entry->goodname, 255);
+        ROM_SETTINGS.goodname[255] = '\0';
+        ROM_SETTINGS.savetype = entry->savetype;
+        ROM_SETTINGS.status = entry->status;
+        ROM_SETTINGS.players = entry->players;
+        ROM_SETTINGS.rumble = entry->rumble;
+        ROM_SETTINGS.transferpak = entry->transferpak;
+        ROM_SETTINGS.mempak = entry->mempak;
+        ROM_SETTINGS.biopak = entry->biopak;
+        ROM_SETTINGS.countperop = entry->countperop;
+        ROM_SETTINGS.disableextramem = entry->disableextramem;
+        ROM_SETTINGS.sidmaduration = entry->sidmaduration;
+        ROM_SETTINGS.aidmamodifier = entry->aidmamodifier;
+        ROM_PARAMS.cheats = entry->cheats;
+    }
+    else
+    {
+        strcpy(ROM_SETTINGS.goodname, "(unknown disk)");
+        /* There's no way to guess the save type, but 4K EEPROM is better than nothing */
+        ROM_SETTINGS.savetype = SAVETYPE_EEPROM_4K;
+        ROM_SETTINGS.status = 0;
+        ROM_SETTINGS.players = 4;
+        ROM_SETTINGS.rumble = 1;
+        ROM_SETTINGS.transferpak = 0;
+        ROM_SETTINGS.mempak = 1;
+        ROM_SETTINGS.biopak = 0;
+        ROM_SETTINGS.countperop = DEFAULT_COUNT_PER_OP;
+        ROM_SETTINGS.disableextramem = DEFAULT_DISABLE_EXTRA_MEM;
+        ROM_SETTINGS.sidmaduration = DEFAULT_SI_DMA_DURATION;
+        ROM_SETTINGS.aidmamodifier = DEFAULT_AI_DMA_MODIFIER;
+        ROM_PARAMS.cheats = NULL;
+    }
+
+    /* set system type */
+    ROM_PARAMS.systemtype = SYSTEM_NTSC;
+
+    /* clear rom header & size */
+    memset(&ROM_HEADER, 0, sizeof(m64p_rom_header));
+    memset(ROM_PARAMS.headername, 0, 20);
+    g_rom_size = 0;
+
+    close_file_storage(fstorage);
+    free(fstorage);
+    return M64ERR_SUCCESS;
+
+wrong_disk_format:
+    close_file_storage(fstorage);
+    dd_disk_filename = NULL; /* already freed in close_file_storage */
+free_fstorage:
+    free(fstorage);
+no_disk:
+    if (dd_disk_filename != NULL) {
+        free(dd_disk_filename);
+    }
+
+    DebugMessage(M64MSG_ERROR, "open_disk(): not a valid disk image");
+    return M64ERR_INPUT_INVALID;
+}
+
+m64p_error close_disk(void)
+{
+    DebugMessage(M64MSG_STATUS, "Disk closed.");
+    return M64ERR_SUCCESS;
+}
+
 /********************************************************************************************/
 /* ROM utility functions */
 
 // Get the system type associated to a ROM country code.
 static m64p_system_type rom_country_code_to_system_type(uint16_t country_code)
 {
-    switch (country_code & UINT16_C(0xFF))
+    switch (country_code)
     {
         // PAL codes
         case 0x44:
@@ -280,33 +413,34 @@ static m64p_system_type rom_country_code_to_system_type(uint16_t country_code)
     }
 }
 
-// Get the VI (vertical interrupt) limit associated to a ROM system type.
-static int rom_system_type_to_vi_limit(m64p_system_type system_type)
+// Converts the homebrew advanced rom header savetype to a m64p_rom_save_type
+static unsigned char rom_homebrew_savetype_to_savetype(uint8_t save_type)
 {
-    switch (system_type)
-    {
-        case SYSTEM_PAL:
-        case SYSTEM_MPAL:
-            return 50;
+    unsigned char m64p_save_type;
 
-        case SYSTEM_NTSC:
-        default:
-            return 60;
-    }
-}
-
-static int rom_system_type_to_ai_dac_rate(m64p_system_type system_type)
-{
-    switch (system_type)
+    switch (save_type)
     {
-        case SYSTEM_PAL:
-            return 49656530;
-        case SYSTEM_MPAL:
-            return 48628316;
-        case SYSTEM_NTSC:
         default:
-            return 48681812;
+        case 0: /* None */
+            m64p_save_type = SAVETYPE_NONE;
+            break;
+        case 1: /* 4K EEPROM */
+            m64p_save_type = SAVETYPE_EEPROM_4K;
+            break;
+        case 2: /* 16K EEPROM */
+            m64p_save_type = SAVETYPE_EEPROM_16KB;
+            break;
+        case 3: /* 256K SRAM */
+        case 4: /* 768K SRAM (banked) */
+        case 6: /* 1M SRAM */
+            m64p_save_type = SAVETYPE_SRAM;
+            break;
+        case 5: /* Flash RAM */
+            m64p_save_type = SAVETYPE_FLASH_RAM;
+            break;
     }
+
+    return m64p_save_type;
 }
 
 static size_t romdatabase_resolve_round(void)
@@ -383,6 +517,42 @@ static size_t romdatabase_resolve_round(void)
             entry->entry.set_flags |= ROMDATABASE_ENTRY_CHEATS;
         }
 
+        if (!isset_bitmask(entry->entry.set_flags, ROMDATABASE_ENTRY_EXTRAMEM) &&
+            isset_bitmask(ref->set_flags, ROMDATABASE_ENTRY_EXTRAMEM)) {
+            entry->entry.disableextramem = ref->disableextramem;
+            entry->entry.set_flags |= ROMDATABASE_ENTRY_EXTRAMEM;
+        }
+
+        if (!isset_bitmask(entry->entry.set_flags, ROMDATABASE_ENTRY_TRANSFERPAK) &&
+            isset_bitmask(ref->set_flags, ROMDATABASE_ENTRY_TRANSFERPAK)) {
+            entry->entry.transferpak = ref->transferpak;
+            entry->entry.set_flags |= ROMDATABASE_ENTRY_TRANSFERPAK;
+        }
+
+        if (!isset_bitmask(entry->entry.set_flags, ROMDATABASE_ENTRY_MEMPAK) &&
+            isset_bitmask(ref->set_flags, ROMDATABASE_ENTRY_MEMPAK)) {
+            entry->entry.mempak = ref->mempak;
+            entry->entry.set_flags |= ROMDATABASE_ENTRY_MEMPAK;
+        }
+
+        if (!isset_bitmask(entry->entry.set_flags, ROMDATABASE_ENTRY_BIOPAK) &&
+            isset_bitmask(ref->set_flags, ROMDATABASE_ENTRY_BIOPAK)) {
+            entry->entry.biopak = ref->biopak;
+            entry->entry.set_flags |= ROMDATABASE_ENTRY_BIOPAK;
+        }
+
+        if (!isset_bitmask(entry->entry.set_flags, ROMDATABASE_ENTRY_SIDMADURATION) &&
+            isset_bitmask(ref->set_flags, ROMDATABASE_ENTRY_SIDMADURATION)) {
+            entry->entry.sidmaduration = ref->sidmaduration;
+            entry->entry.set_flags |= ROMDATABASE_ENTRY_SIDMADURATION;
+        }
+
+        if (!isset_bitmask(entry->entry.set_flags, ROMDATABASE_ENTRY_AIDMAMODIFIER) &&
+            isset_bitmask(ref->set_flags, ROMDATABASE_ENTRY_AIDMAMODIFIER)) {
+            entry->entry.aidmamodifier = ref->aidmamodifier;
+            entry->entry.set_flags |= ROMDATABASE_ENTRY_AIDMAMODIFIER;
+        }
+
         free(entry->entry.refmd5);
         entry->entry.refmd5 = NULL;
     }
@@ -423,7 +593,7 @@ void romdatabase_open(void)
         return;
 
     /* Open romdatabase. */
-    if (pathname == NULL || (fPtr = fopen(pathname, "rb")) == NULL)
+    if (pathname == NULL || (fPtr = osal_file_open(pathname, "rb")) == NULL)
     {
         DebugMessage(M64MSG_ERROR, "Unable to open rom database file '%s'.", pathname);
         return;
@@ -457,21 +627,29 @@ void romdatabase_open(void)
                 continue;
             }
 
-            *next_search = (romdatabase_search*)malloc(sizeof(romdatabase_search));
+            *next_search = (romdatabase_search*) malloc(sizeof(romdatabase_search));
             search = *next_search;
             next_search = &search->next_entry;
 
+            memset(search, 0, sizeof(romdatabase_search));
+
             search->entry.goodname = NULL;
             memcpy(search->entry.md5, md5, 16);
             search->entry.refmd5 = NULL;
             search->entry.crc1 = 0;
             search->entry.crc2 = 0;
             search->entry.status = 0; /* Set default to 0 stars. */
-            search->entry.savetype = DEFAULT;
-            search->entry.players = DEFAULT;
-            search->entry.rumble = DEFAULT; 
-            search->entry.countperop = COUNT_PER_OP_DEFAULT;
+            search->entry.savetype = SAVETYPE_EEPROM_4K;
+            search->entry.players = 4;
+            search->entry.rumble = 1;
+            search->entry.countperop = DEFAULT_COUNT_PER_OP;
+            search->entry.disableextramem = DEFAULT_DISABLE_EXTRA_MEM;
             search->entry.cheats = NULL;
+            search->entry.transferpak = 0;
+            search->entry.mempak = 1;
+            search->entry.biopak = 0;
+            search->entry.sidmaduration = DEFAULT_SI_DMA_DURATION;
+            search->entry.aidmamodifier = DEFAULT_AI_DMA_MODIFIER;
             search->entry.set_flags = ROMDATABASE_ENTRY_NONE;
 
             search->next_entry = NULL;
@@ -528,22 +706,22 @@ void romdatabase_open(void)
             else if(!strcmp(l.name, "SaveType"))
             {
                 if(!strcmp(l.value, "Eeprom 4KB")) {
-                    search->entry.savetype = EEPROM_4KB;
+                    search->entry.savetype = SAVETYPE_EEPROM_4K;
                     search->entry.set_flags |= ROMDATABASE_ENTRY_SAVETYPE;
                 } else if(!strcmp(l.value, "Eeprom 16KB")) {
-                    search->entry.savetype = EEPROM_16KB;
+                    search->entry.savetype = SAVETYPE_EEPROM_16K;
                     search->entry.set_flags |= ROMDATABASE_ENTRY_SAVETYPE;
                 } else if(!strcmp(l.value, "SRAM")) {
-                    search->entry.savetype = SRAM;
+                    search->entry.savetype = SAVETYPE_SRAM;
                     search->entry.set_flags |= ROMDATABASE_ENTRY_SAVETYPE;
                 } else if(!strcmp(l.value, "Flash RAM")) {
-                    search->entry.savetype = FLASH_RAM;
+                    search->entry.savetype = SAVETYPE_FLASH_RAM;
                     search->entry.set_flags |= ROMDATABASE_ENTRY_SAVETYPE;
                 } else if(!strcmp(l.value, "Controller Pack")) {
-                    search->entry.savetype = CONTROLLER_PACK;
+                    search->entry.savetype = SAVETYPE_CONTROLLER_PAK;
                     search->entry.set_flags |= ROMDATABASE_ENTRY_SAVETYPE;
                 } else if(!strcmp(l.value, "None")) {
-                    search->entry.savetype = NONE;
+                    search->entry.savetype = SAVETYPE_NONE;
                     search->entry.set_flags |= ROMDATABASE_ENTRY_SAVETYPE;
                 } else {
                     DebugMessage(M64MSG_WARNING, "ROM Database: Invalid save type on line %i", lineno);
@@ -588,6 +766,11 @@ void romdatabase_open(void)
                     DebugMessage(M64MSG_WARNING, "ROM Database: Invalid CountPerOp on line %i", lineno);
                 }
             }
+            else if (!strcmp(l.name, "DisableExtraMem"))
+            {
+                search->entry.disableextramem = atoi(l.value);
+                search->entry.set_flags |= ROMDATABASE_ENTRY_EXTRAMEM;
+            }
             else if(!strncmp(l.name, "Cheat", 5))
             {
                 size_t len1 = 0, len2 = 0;
@@ -618,6 +801,60 @@ void romdatabase_open(void)
 
                 search->entry.set_flags |= ROMDATABASE_ENTRY_CHEATS;
             }
+            else if(!strcmp(l.name, "Transferpak"))
+            {
+                if(!strcmp(l.value, "Yes")) {
+                    search->entry.transferpak = 1;
+                    search->entry.set_flags |= ROMDATABASE_ENTRY_TRANSFERPAK;
+                } else if(!strcmp(l.value, "No")) {
+                    search->entry.transferpak = 0;
+                    search->entry.set_flags |= ROMDATABASE_ENTRY_TRANSFERPAK;
+                } else {
+                    DebugMessage(M64MSG_WARNING, "ROM Database: Invalid transferpak string on line %i", lineno);
+                }
+            }
+            else if(!strcmp(l.name, "Mempak"))
+            {
+                if(!strcmp(l.value, "Yes")) {
+                    search->entry.mempak = 1;
+                    search->entry.set_flags |= ROMDATABASE_ENTRY_MEMPAK;
+                } else if(!strcmp(l.value, "No")) {
+                    search->entry.mempak = 0;
+                    search->entry.set_flags |= ROMDATABASE_ENTRY_MEMPAK;
+                } else {
+                    DebugMessage(M64MSG_WARNING, "ROM Database: Invalid mempak string on line %i", lineno);
+                }
+            }
+            else if(!strcmp(l.name, "Biopak"))
+            {
+                if(!strcmp(l.value, "Yes")) {
+                    search->entry.biopak = 1;
+                    search->entry.set_flags |= ROMDATABASE_ENTRY_BIOPAK;
+                } else if(!strcmp(l.value, "No")) {
+                    search->entry.biopak = 0;
+                    search->entry.set_flags |= ROMDATABASE_ENTRY_BIOPAK;
+                } else {
+                    DebugMessage(M64MSG_WARNING, "ROM Database: Invalid biopak string on line %i", lineno);
+                }
+            }
+            else if(!strcmp(l.name, "SiDmaDuration"))
+            {
+                if (string_to_int(l.value, &value) && value >= 0 && value <= 0x10000) {
+                    search->entry.sidmaduration = value;
+                    search->entry.set_flags |= ROMDATABASE_ENTRY_SIDMADURATION;
+                } else {
+                    DebugMessage(M64MSG_WARNING, "ROM Database: Invalid SiDmaDuration on line %i", lineno);
+                }
+            }
+            else if(!strcmp(l.name, "AiDmaModifier"))
+            {
+                if (string_to_int(l.value, &value) && value >= 0 && value <= 200) {
+                    search->entry.aidmamodifier = value;
+                    search->entry.set_flags |= ROMDATABASE_ENTRY_AIDMAMODIFIER;
+                } else {
+                    DebugMessage(M64MSG_WARNING, "ROM Database: Invalid AiDmaModifier on line %i", lineno);
+                }
+            }
             else
             {
                 DebugMessage(M64MSG_WARNING, "ROM Database: Unknown property on line %i", lineno);
@@ -648,6 +885,8 @@ void romdatabase_close(void)
         free(g_romdatabase.list);
         g_romdatabase.list = search;
         }
+
+    g_romdatabase.have_database = 0;
 }
 
 static romdatabase_entry* ini_search_by_md5(md5_byte_t* md5)
@@ -671,19 +910,28 @@ static romdatabase_entry* ini_search_by_md5(md5_byte_t* md5)
 romdatabase_entry* ini_search_by_crc(unsigned int crc1, unsigned int crc2)
 {
     romdatabase_search* search;
+    romdatabase_entry* found_entry = NULL;
 
     if(!g_romdatabase.have_database) 
         return NULL;
 
     search = g_romdatabase.crc_lists[((crc1 >> 24) & 0xff)];
 
-    while (search != NULL && search->entry.crc1 != crc1 && search->entry.crc2 != crc2)
+    // because CRCs can be ambiguous (there can be multiple database entries with the same CRC),
+    // we will prefer MD5 hashes instead. If the given CRC matches more than one entry in the
+    // database, we will return no match.
+    while (search != NULL)
+    {
+        if (search->entry.crc1 == crc1 && search->entry.crc2 == crc2)
+        {
+            if (found_entry != NULL)
+                return NULL;
+            found_entry = &search->entry;
+        }
         search = search->next_crc;
+    }
 
-    if(search == NULL) 
-        return NULL;
-
-    return &(search->entry);
+    return found_entry;
 }
 
 
