@@ -1,6 +1,6 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  *   Mupen64plus - plugin.c                                                *
- *   Mupen64Plus homepage: http://code.google.com/p/mupen64plus/           *
+ *   Mupen64Plus homepage: https://mupen64plus.org/                        *
  *   Copyright (C) 2002 Hacktarux                                          *
  *   Copyright (C) 2009 Richard Goedeken                                   *
  *                                                                         *
@@ -22,12 +22,18 @@
 
 #include <stdint.h>
 #include <stdlib.h>
+#include <string.h>
 
-#include "ai/ai_controller.h"
 #include "api/callbacks.h"
 #include "api/m64p_common.h"
 #include "api/m64p_plugin.h"
 #include "api/m64p_types.h"
+#include "device/memory/memory.h"
+#include "device/rcp/ai/ai_controller.h"
+#include "device/rcp/mi/mi_controller.h"
+#include "device/rcp/rdp/rdp_core.h"
+#include "device/rcp/rsp/rsp_core.h"
+#include "device/rcp/vi/vi_controller.h"
 #include "dummy_audio.h"
 #include "dummy_input.h"
 #include "dummy_rsp.h"
@@ -35,13 +41,8 @@
 #include "main/main.h"
 #include "main/rom.h"
 #include "main/version.h"
-#include "memory/memory.h"
 #include "osal/dynamiclib.h"
 #include "plugin.h"
-#include "r4300/r4300_core.h"
-#include "rdp/rdp_core.h"
-#include "rsp/rsp_core.h"
-#include "vi/vi_controller.h"
 
 CONTROL Controls[4];
 
@@ -99,7 +100,8 @@ static const input_plugin_functions dummy_input = {
     dummyinput_RomClosed,
     dummyinput_RomOpen,
     dummyinput_SDL_KeyDown,
-    dummyinput_SDL_KeyUp
+    dummyinput_SDL_KeyUp,
+    dummyinput_RenderCallback
 };
 
 static const rsp_plugin_functions dummy_rsp = {
@@ -190,7 +192,7 @@ static m64p_error plugin_connect_gfx(m64p_dynlib_handle plugin_handle)
         }
 
         /* set function pointers for optional functions */
-        gfx.resizeVideoOutput = (ptr_ResizeVideoOutput) osal_dynlib_getproc(plugin_handle, "ResizeVideoOutput");
+        gfx.resizeVideoOutput = (ptr_ResizeVideoOutput)osal_dynlib_getproc(plugin_handle, "ResizeVideoOutput");
 
         /* check the version info */
         (*gfx.getVersion)(&PluginType, &PluginVersion, &APIVersion, NULL, NULL);
@@ -226,36 +228,57 @@ static m64p_error plugin_connect_gfx(m64p_dynlib_handle plugin_handle)
 
 static m64p_error plugin_start_gfx(void)
 {
+    uint8_t media = *((uint8_t*)mem_base_u32(g_mem_base, MM_CART_ROM) + (0x3b ^ S8));
+
+    /* Here we feed 64DD IPL ROM header to GFX plugin if 64DD is present.
+     * We use g_media_loader.get_dd_rom to detect 64DD presence
+     * instead of g_dev because the latter is not yet initialized at plugin_start time */
+    /* XXX: Not sure it is the best way to convey which game is being played to the GFX plugin
+     * as 64DD IPL is the same for all 64DD games... */
+    char* dd_ipl_rom_filename = (g_media_loader.get_dd_rom == NULL)
+        ? NULL
+        : g_media_loader.get_dd_rom(g_media_loader.cb_data);
+
+    uint32_t rom_base = (g_rom_size == 0 || (dd_ipl_rom_filename != NULL && strlen(dd_ipl_rom_filename) != 0 && media != 'C'))
+        ? MM_DD_ROM
+        : MM_CART_ROM;
+
+    free(dd_ipl_rom_filename);
+
     /* fill in the GFX_INFO data structure */
-    gfx_info.HEADER = (unsigned char *) g_rom;
-    gfx_info.RDRAM = (unsigned char *) g_rdram;
-    gfx_info.DMEM = (unsigned char *) g_sp.mem;
-    gfx_info.IMEM = (unsigned char *) g_sp.mem + 0x1000;
-    gfx_info.MI_INTR_REG = &(g_r4300.mi.regs[MI_INTR_REG]);
-    gfx_info.DPC_START_REG = &(g_dp.dpc_regs[DPC_START_REG]);
-    gfx_info.DPC_END_REG = &(g_dp.dpc_regs[DPC_END_REG]);
-    gfx_info.DPC_CURRENT_REG = &(g_dp.dpc_regs[DPC_CURRENT_REG]);
-    gfx_info.DPC_STATUS_REG = &(g_dp.dpc_regs[DPC_STATUS_REG]);
-    gfx_info.DPC_CLOCK_REG = &(g_dp.dpc_regs[DPC_CLOCK_REG]);
-    gfx_info.DPC_BUFBUSY_REG = &(g_dp.dpc_regs[DPC_BUFBUSY_REG]);
-    gfx_info.DPC_PIPEBUSY_REG = &(g_dp.dpc_regs[DPC_PIPEBUSY_REG]);
-    gfx_info.DPC_TMEM_REG = &(g_dp.dpc_regs[DPC_TMEM_REG]);
-    gfx_info.VI_STATUS_REG = &(g_vi.regs[VI_STATUS_REG]);
-    gfx_info.VI_ORIGIN_REG = &(g_vi.regs[VI_ORIGIN_REG]);
-    gfx_info.VI_WIDTH_REG = &(g_vi.regs[VI_WIDTH_REG]);
-    gfx_info.VI_INTR_REG = &(g_vi.regs[VI_V_INTR_REG]);
-    gfx_info.VI_V_CURRENT_LINE_REG = &(g_vi.regs[VI_CURRENT_REG]);
-    gfx_info.VI_TIMING_REG = &(g_vi.regs[VI_BURST_REG]);
-    gfx_info.VI_V_SYNC_REG = &(g_vi.regs[VI_V_SYNC_REG]);
-    gfx_info.VI_H_SYNC_REG = &(g_vi.regs[VI_H_SYNC_REG]);
-    gfx_info.VI_LEAP_REG = &(g_vi.regs[VI_LEAP_REG]);
-    gfx_info.VI_H_START_REG = &(g_vi.regs[VI_H_START_REG]);
-    gfx_info.VI_V_START_REG = &(g_vi.regs[VI_V_START_REG]);
-    gfx_info.VI_V_BURST_REG = &(g_vi.regs[VI_V_BURST_REG]);
-    gfx_info.VI_X_SCALE_REG = &(g_vi.regs[VI_X_SCALE_REG]);
-    gfx_info.VI_Y_SCALE_REG = &(g_vi.regs[VI_Y_SCALE_REG]);
+    gfx_info.HEADER = (unsigned char *)mem_base_u32(g_mem_base, rom_base);
+    gfx_info.RDRAM = (unsigned char *)mem_base_u32(g_mem_base, MM_RDRAM_DRAM);
+    gfx_info.DMEM = (unsigned char *)mem_base_u32(g_mem_base, MM_RSP_MEM);
+    gfx_info.IMEM = (unsigned char *)mem_base_u32(g_mem_base, MM_RSP_MEM + 0x1000);
+    gfx_info.MI_INTR_REG = &(g_dev.mi.regs[MI_INTR_REG]);
+    gfx_info.DPC_START_REG = &(g_dev.dp.dpc_regs[DPC_START_REG]);
+    gfx_info.DPC_END_REG = &(g_dev.dp.dpc_regs[DPC_END_REG]);
+    gfx_info.DPC_CURRENT_REG = &(g_dev.dp.dpc_regs[DPC_CURRENT_REG]);
+    gfx_info.DPC_STATUS_REG = &(g_dev.dp.dpc_regs[DPC_STATUS_REG]);
+    gfx_info.DPC_CLOCK_REG = &(g_dev.dp.dpc_regs[DPC_CLOCK_REG]);
+    gfx_info.DPC_BUFBUSY_REG = &(g_dev.dp.dpc_regs[DPC_BUFBUSY_REG]);
+    gfx_info.DPC_PIPEBUSY_REG = &(g_dev.dp.dpc_regs[DPC_PIPEBUSY_REG]);
+    gfx_info.DPC_TMEM_REG = &(g_dev.dp.dpc_regs[DPC_TMEM_REG]);
+    gfx_info.VI_STATUS_REG = &(g_dev.vi.regs[VI_STATUS_REG]);
+    gfx_info.VI_ORIGIN_REG = &(g_dev.vi.regs[VI_ORIGIN_REG]);
+    gfx_info.VI_WIDTH_REG = &(g_dev.vi.regs[VI_WIDTH_REG]);
+    gfx_info.VI_INTR_REG = &(g_dev.vi.regs[VI_V_INTR_REG]);
+    gfx_info.VI_V_CURRENT_LINE_REG = &(g_dev.vi.regs[VI_CURRENT_REG]);
+    gfx_info.VI_TIMING_REG = &(g_dev.vi.regs[VI_BURST_REG]);
+    gfx_info.VI_V_SYNC_REG = &(g_dev.vi.regs[VI_V_SYNC_REG]);
+    gfx_info.VI_H_SYNC_REG = &(g_dev.vi.regs[VI_H_SYNC_REG]);
+    gfx_info.VI_LEAP_REG = &(g_dev.vi.regs[VI_LEAP_REG]);
+    gfx_info.VI_H_START_REG = &(g_dev.vi.regs[VI_H_START_REG]);
+    gfx_info.VI_V_START_REG = &(g_dev.vi.regs[VI_V_START_REG]);
+    gfx_info.VI_V_BURST_REG = &(g_dev.vi.regs[VI_V_BURST_REG]);
+    gfx_info.VI_X_SCALE_REG = &(g_dev.vi.regs[VI_X_SCALE_REG]);
+    gfx_info.VI_Y_SCALE_REG = &(g_dev.vi.regs[VI_Y_SCALE_REG]);
     gfx_info.CheckInterrupts = EmptyFunc;
 
+    gfx_info.version = 2; //Version 2 added SP_STATUS_REG and RDRAM_SIZE
+    gfx_info.SP_STATUS_REG = &g_dev.sp.regs[SP_STATUS_REG];
+    gfx_info.RDRAM_SIZE = (unsigned int*) &g_dev.rdram.dram_size;
+
     /* call the audio plugin */
     if (!gfx.initiateGFX(gfx_info))
         return M64ERR_PLUGIN_FAIL;
@@ -320,16 +343,16 @@ static m64p_error plugin_connect_audio(m64p_dynlib_handle plugin_handle)
 static m64p_error plugin_start_audio(void)
 {
     /* fill in the AUDIO_INFO data structure */
-    audio_info.RDRAM = (unsigned char *) g_rdram;
-    audio_info.DMEM = (unsigned char *) g_sp.mem;
-    audio_info.IMEM = (unsigned char *) g_sp.mem + 0x1000;
-    audio_info.MI_INTR_REG = &(g_r4300.mi.regs[MI_INTR_REG]);
-    audio_info.AI_DRAM_ADDR_REG = &(g_ai.regs[AI_DRAM_ADDR_REG]);
-    audio_info.AI_LEN_REG = &(g_ai.regs[AI_LEN_REG]);
-    audio_info.AI_CONTROL_REG = &(g_ai.regs[AI_CONTROL_REG]);
+    audio_info.RDRAM = (unsigned char *)mem_base_u32(g_mem_base, MM_RDRAM_DRAM);
+    audio_info.DMEM = (unsigned char *)mem_base_u32(g_mem_base, MM_RSP_MEM);
+    audio_info.IMEM = (unsigned char *)mem_base_u32(g_mem_base, MM_RSP_MEM + 0x1000);
+    audio_info.MI_INTR_REG = &(g_dev.mi.regs[MI_INTR_REG]);
+    audio_info.AI_DRAM_ADDR_REG = &(g_dev.ai.regs[AI_DRAM_ADDR_REG]);
+    audio_info.AI_LEN_REG = &(g_dev.ai.regs[AI_LEN_REG]);
+    audio_info.AI_CONTROL_REG = &(g_dev.ai.regs[AI_CONTROL_REG]);
     audio_info.AI_STATUS_REG = &dummy;
-    audio_info.AI_DACRATE_REG = &(g_ai.regs[AI_DACRATE_REG]);
-    audio_info.AI_BITRATE_REG = &(g_ai.regs[AI_BITRATE_REG]);
+    audio_info.AI_DACRATE_REG = &(g_dev.ai.regs[AI_DACRATE_REG]);
+    audio_info.AI_BITRATE_REG = &(g_dev.ai.regs[AI_BITRATE_REG]);
     audio_info.CheckInterrupts = EmptyFunc;
 
     /* call the audio plugin */
@@ -371,25 +394,27 @@ static m64p_error plugin_connect_input(m64p_dynlib_handle plugin_handle)
             return M64ERR_INPUT_INVALID;
         }
 
+        if (!GET_FUNC(ptr_SendVRUWord, input.sendVRUWord, "SendVRUWord") ||
+            !GET_FUNC(ptr_SetMicState, input.setMicState, "SetMicState") ||
+            !GET_FUNC(ptr_ReadVRUResults, input.readVRUResults, "ReadVRUResults") ||
+            !GET_FUNC(ptr_ClearVRUWords, input.clearVRUWords, "ClearVRUWords") ||
+            !GET_FUNC(ptr_SetVRUWordMask, input.setVRUWordMask, "SetVRUWordMask"))
+        {
+            DebugMessage(M64MSG_WARNING, "Input plugin does not contain VRU support.");
+        }
+
         /* check the version info */
         (*input.getVersion)(&PluginType, &PluginVersion, &APIVersion, NULL, NULL);
-        if (PluginType != M64PLUGIN_INPUT || (APIVersion & 0xffff0000) != (INPUT_API_VERSION & 0xffff0000))
+        if (PluginType != M64PLUGIN_INPUT || (APIVersion & 0xffff0000) != (INPUT_API_VERSION & 0xffff0000) || APIVersion < 0x020100)
         {
             DebugMessage(M64MSG_ERROR, "incompatible Input plugin");
             plugin_disconnect_input();
             return M64ERR_INCOMPATIBLE;
         }
 
-        if (APIVersion < 0x020001)
+        if (!GET_FUNC(ptr_RenderCallback, input.renderCallback, "RenderCallback"))
         {
-            input.renderCallback = NULL; // not supported in earlier input plugins
-        }
-        else
-        {
-            if (!GET_FUNC(ptr_RenderCallback, input.renderCallback, "RenderCallback"))
-            {
-                DebugMessage(M64MSG_INFO, "input plugin did not specify a render callback; there will be no on screen display by the input plugin.");
-            }
+            DebugMessage(M64MSG_INFO, "input plugin did not specify a render callback; there will be no on screen display by the input plugin.");
         }
 
         l_InputAttached = 1;
@@ -411,6 +436,7 @@ static m64p_error plugin_start_input(void)
          Controls[i].Present = 0;
          Controls[i].RawData = 0;
          Controls[i].Plugin = PLUGIN_NONE;
+         Controls[i].Type = CONT_TYPE_STANDARD;
       }
 
     /* call the input plugin */
@@ -466,27 +492,27 @@ static m64p_error plugin_connect_rsp(m64p_dynlib_handle plugin_handle)
 static m64p_error plugin_start_rsp(void)
 {
     /* fill in the RSP_INFO data structure */
-    rsp_info.RDRAM = (unsigned char *) g_rdram;
-    rsp_info.DMEM = (unsigned char *) g_sp.mem;
-    rsp_info.IMEM = (unsigned char *) g_sp.mem + 0x1000;
-    rsp_info.MI_INTR_REG = &g_r4300.mi.regs[MI_INTR_REG];
-    rsp_info.SP_MEM_ADDR_REG = &g_sp.regs[SP_MEM_ADDR_REG];
-    rsp_info.SP_DRAM_ADDR_REG = &g_sp.regs[SP_DRAM_ADDR_REG];
-    rsp_info.SP_RD_LEN_REG = &g_sp.regs[SP_RD_LEN_REG];
-    rsp_info.SP_WR_LEN_REG = &g_sp.regs[SP_WR_LEN_REG];
-    rsp_info.SP_STATUS_REG = &g_sp.regs[SP_STATUS_REG];
-    rsp_info.SP_DMA_FULL_REG = &g_sp.regs[SP_DMA_FULL_REG];
-    rsp_info.SP_DMA_BUSY_REG = &g_sp.regs[SP_DMA_BUSY_REG];
-    rsp_info.SP_PC_REG = &g_sp.regs2[SP_PC_REG];
-    rsp_info.SP_SEMAPHORE_REG = &g_sp.regs[SP_SEMAPHORE_REG];
-    rsp_info.DPC_START_REG = &g_dp.dpc_regs[DPC_START_REG];
-    rsp_info.DPC_END_REG = &g_dp.dpc_regs[DPC_END_REG];
-    rsp_info.DPC_CURRENT_REG = &g_dp.dpc_regs[DPC_CURRENT_REG];
-    rsp_info.DPC_STATUS_REG = &g_dp.dpc_regs[DPC_STATUS_REG];
-    rsp_info.DPC_CLOCK_REG = &g_dp.dpc_regs[DPC_CLOCK_REG];
-    rsp_info.DPC_BUFBUSY_REG = &g_dp.dpc_regs[DPC_BUFBUSY_REG];
-    rsp_info.DPC_PIPEBUSY_REG = &g_dp.dpc_regs[DPC_PIPEBUSY_REG];
-    rsp_info.DPC_TMEM_REG = &g_dp.dpc_regs[DPC_TMEM_REG];
+    rsp_info.RDRAM = (unsigned char *)mem_base_u32(g_mem_base, MM_RDRAM_DRAM);
+    rsp_info.DMEM = (unsigned char *)mem_base_u32(g_mem_base, MM_RSP_MEM);
+    rsp_info.IMEM = (unsigned char *)mem_base_u32(g_mem_base, MM_RSP_MEM + 0x1000);
+    rsp_info.MI_INTR_REG = &g_dev.mi.regs[MI_INTR_REG];
+    rsp_info.SP_MEM_ADDR_REG = &g_dev.sp.regs[SP_MEM_ADDR_REG];
+    rsp_info.SP_DRAM_ADDR_REG = &g_dev.sp.regs[SP_DRAM_ADDR_REG];
+    rsp_info.SP_RD_LEN_REG = &g_dev.sp.regs[SP_RD_LEN_REG];
+    rsp_info.SP_WR_LEN_REG = &g_dev.sp.regs[SP_WR_LEN_REG];
+    rsp_info.SP_STATUS_REG = &g_dev.sp.regs[SP_STATUS_REG];
+    rsp_info.SP_DMA_FULL_REG = &g_dev.sp.regs[SP_DMA_FULL_REG];
+    rsp_info.SP_DMA_BUSY_REG = &g_dev.sp.regs[SP_DMA_BUSY_REG];
+    rsp_info.SP_PC_REG = &g_dev.sp.regs2[SP_PC_REG];
+    rsp_info.SP_SEMAPHORE_REG = &g_dev.sp.regs[SP_SEMAPHORE_REG];
+    rsp_info.DPC_START_REG = &g_dev.dp.dpc_regs[DPC_START_REG];
+    rsp_info.DPC_END_REG = &g_dev.dp.dpc_regs[DPC_END_REG];
+    rsp_info.DPC_CURRENT_REG = &g_dev.dp.dpc_regs[DPC_CURRENT_REG];
+    rsp_info.DPC_STATUS_REG = &g_dev.dp.dpc_regs[DPC_STATUS_REG];
+    rsp_info.DPC_CLOCK_REG = &g_dev.dp.dpc_regs[DPC_CLOCK_REG];
+    rsp_info.DPC_BUFBUSY_REG = &g_dev.dp.dpc_regs[DPC_BUFBUSY_REG];
+    rsp_info.DPC_PIPEBUSY_REG = &g_dev.dp.dpc_regs[DPC_PIPEBUSY_REG];
+    rsp_info.DPC_TMEM_REG = &g_dev.dp.dpc_regs[DPC_TMEM_REG];
     rsp_info.CheckInterrupts = EmptyFunc;
     rsp_info.ProcessDlistList = gfx.processDList;
     rsp_info.ProcessAlistList = audio.processAList;
