@@ -1,6 +1,6 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  *   Mupen64plus - r4300prof.c                                             *
- *   Mupen64Plus homepage: http://code.google.com/p/mupen64plus/           *
+ *   Mupen64Plus homepage: https://mupen64plus.org/                        *
  *   Copyright (C) 2008 Richard Goedeken                                   *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -119,7 +119,7 @@ int main(int argc, void *argv[])
 
   /* open r4300 opcode/x86 address table generated from emulator run */
   printf("Loading %s...\n", argv[1]);
-  pfIn = fopen(argv[1], "rb");
+  pfIn = osal_file_open(argv[1], "rb");
   if (pfIn == NULL)
   {
     printf("Couldn't open input file: %s\n", argv[1]);
@@ -168,7 +168,7 @@ int main(int argc, void *argv[])
 
   /* open the profiling sample data file */
   printf("Loading %s...\n", argv[2]);
-  pfIn = fopen(argv[2], "rb");
+  pfIn = osal_file_open(argv[2], "rb");
   if (pfIn == NULL)
   {
     printf("Couldn't open input file: %s\n", argv[2]);
